(* 
 File: PepaNets.sig

 Interface to the internals of the PEPA Workbench [for PEPA Nets].
*)

signature PepaNets =
sig
  type Identifier = HashTable.hashedstring 

  type Name = int

  datatype Rate = 
          Top
        | Rate of Identifier
        | Product of int * Identifier 
 
  datatype Component =  
	  PREFIX     of (Identifier * Rate) * Component
	| CHOICE     of Component * Component
	| COOP       of Component list * Identifier list
	| HIDING     of Component * Identifier list
        | CELL       of Identifier * Component option
	| VAR        of Identifier
        | RATE       of string
	| CONSTS     of Identifier * Component * Component
	| ARC        of Identifier * (Identifier * Rate) * Identifier
	| PRIORITIES of Identifier list list
	| MARKING    of Component list

  (* States and components are isomorphic.  The difference
     is that components are specified by the modeller and
      states are found by the Workbench.
  *)
  type State = Component

  type Environment = (Identifier * Component) list

  (* Firings and transitions are isomorphic as far as the 
     generation of the state space is concerned.
  *)

  type Transition = Component * 
       ((Identifier * Rate) * Name) option * Component

  (* Determine the minimum of two rates *)
  val min : Rate * Rate  -> Rate

end;
